""" Adapted from Python Packaging User Guide. """

from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='sg_helper',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.0.0',

    description='A numpy interface for the SG++ adaptive sparse grids library',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/dgreenwald/sg_helper',

    # Author details
    author='Dan Greenwald',
    author_email='greenw2@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
                # How mature is this project? Common values are
                #   3 - Alpha
                #   4 - Beta
                #   5 - Production/Stable
                'Development Status :: 3 - Alpha',

                # Indicate who your project is intended for
                # 'Intended Audience :: Developers',
                # 'Topic :: Software Development :: Build Tools',

                # Pick your license as you wish (should match "license" above)
                'License :: OSI Approved :: MIT License',

                # Specify the Python versions you support here. In particular, ensure
                # that you indicate whether you support Python 2, Python 3 or both.
                'Programming Language :: Python :: 2',
                'Programming Language :: Python :: 2.6',
                'Programming Language :: Python :: 2.7',
                # 'Programming Language :: Python :: 3',
                # 'Programming Language :: Python :: 3.3',
                # 'Programming Language :: Python :: 3.4',
                # 'Programming Language :: Python :: 3.5',
            
    ],
    
    # What does your project relate to?
    keywords='adaptive sparse grids',

    packages=find_packages(),
    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    # py_modules=["sg_helper"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['numpy'],
)
